// ComputationalSpecificationSet.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;


/**
 * The container generated by an order and containing the spefications about
 * a computational process
 *
 * @author Pietro Terna
 */
public class ComputationalSpecificationSet  extends SwarmObjectImpl{

    /** myOrder (i.e. that of the order which has generated this object */
    public Order myOrder;
    /** the position of this sequence in the recipe */
    public int pos;
    /** the specifications (as addresses of memory matrixes to be used) */
    public Object [] matrixAddress;
    /** setting true when done */
    public boolean done=false;
    /** the tipe of computation (max -1001, min -1999) */
    public int computationType;

  /**
   * constructor for computational specification set
   */
  public ComputationalSpecificationSet (Zone aZone, Order o, int p) {

     // Call the constructor for the parent class
     super(aZone);

     myOrder=o;
     pos=p;

     if(StartVEFrame.verbose)
	 System.out.println("Comp. spec. created " +
			    "(by order " + getOrderNumber() 
			    + " at pos. " + pos + "; NB first pos. is 0)");
  }

    /** setting computational specifications */
    public void setSpecificationSet(int n, Object [] r, int ct)
    {
	int i;
	// the vector containing 'n' memory matrixes addresses
	matrixAddress = new Object[n];

	for (i=0; i<n; i++) 
	    matrixAddress[i] = r[i];

	computationType=ct;

	if(StartVEFrame.verbose)
	    System.out.println("Number of memory matrixes to be used = " + n +
			       " making computation " + computationType);
    }

    /** returning the number of memory matrixes to be used */
    public int getNumberOfMemoryMatrixesToBeUsed(){
	return matrixAddress.length;
    }

    /** returning the memory matrix address at pos. i of the vector 
     *  containg the addresses (0<=i<matrixAddress.length) */
    public Object getMemoryMatrixAddress(int i){
	return matrixAddress[i];
    }

    /**
     * returning the number of the order of this computational 
     * specification set
     */
    public int getOrderNumber () {
	return myOrder.getOrderNumber();
   }

    /**
     * returning the position of this computational specification set
     * into its recipe
     */
    public int getPositionInRecipe() {
	return pos;
   }

    /** setting the computation done */
    public void setDone(){
	done=true;
    }

    /** getting the done status */
    public boolean getDone(){
	return done;
    }

    /** getting the order layer */
    public int getOrderLayer(){
	return myOrder.getOrderLayer();
    }

    /** getting the computation type */
    public int getComputationType(){
	return computationType;
    }

}
