// Or.java

import swarm.Globals;
import swarm.collections.ListImpl;

/**
 * An Or manager for our orders<br><br>
 *
 * orCriterion<br>
 *			0 - all 'or' branches in sequence<br>
 *			1 - choosing first branch<br>
 *			2 - choosing second branch<br>
 *			3 - a random branch<br>
 *			4 - the branch whose first step
 *			    has the shortest waiting list<br><br>
 *
 * @author Pietro Terna
 */
public class Or
{
    // the class is used in a static way, so we have no construtors 
    // here and we have to declare static all the methods
 
    public static int orCriterion;
    /** the list of the operating units */
    public static ListImpl unitList;

    public static void setOrCriterion(int k){
	orCriterion=k;
    }

    public static void setUnitList(ListImpl ul){
	unitList=ul;
    }

    public static void applyOr(Order o){
	OrSpecificationSet spec; 
	Unit aUnit;
	boolean unitNotFound;
	int i, iii, node, chosenNode, length, length0;
	int [] recipe, state;

	// the or specification set to be used
	spec=(OrSpecificationSet) o.getPendingOrSpecificationSet();
	if(spec == null){
	    System.out.println("Internal error: found an order x in " +
			       "Or.applyOr(x) without 'or' specifications");
	    MyExit.exit(1);
	}

	// applying the 'or' criterion
	if(orCriterion == 0)return;

	chosenNode=0;
	recipe = (int []) o.getRecipeVector();
	state =  (int []) o.getStateVector();

	if(orCriterion == 1)chosenNode=1;
	if(orCriterion == 2)chosenNode=2;
	if(orCriterion == 3)chosenNode=Globals.env.uniformIntRand.
			     getIntegerWithMin$withMax(1,spec.getNodeNumber());

	if(orCriterion == 4)
	    {
		length=1000000000;
		// looking for the first unit in each branch
		for (node=1; node<=spec.getNodeNumber();node++)
		    {
			length0=length;
			unitNotFound=true;
			for (i=0;i<unitList.getCount() && unitNotFound;i++)
			    {
				aUnit=(Unit) unitList.atOffset(i);
				for (iii=0;iii<aUnit.getNOfPhasesToDealWith() 
					 && unitNotFound;iii++)
				    {
					if(recipe[spec.getNodePosition(node)]==
					   aUnit.getProductionPhase(iii))
					    {
						unitNotFound=false;
						length=aUnit.
						    getWaitingListLength();
						if(length<length0)chosenNode=
								      node;
					    }
				    }
			    }
			
		    }
	    }

	if(StartVEFrame.verbose)
	    System.out.println("Applying Or to order number "+
			       o.getOrderNumber()+
			       " at pos. "+spec.getPositionInRecipe());

	for (node=1; node<=spec.getNodeNumber();node++)
	    {
		for(i=spec.getNodePosition(node);
		    i<spec.getNodePosition(node+1);i++)
		    {
			if(chosenNode != node) state[i]=-1;
		    }
	    }

    }

}
