//  ProcurementAssembler.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

import swarm.collections.ListImpl;
import swarm.collections.ListIndex;

/**
 * The ProcurementAssembler class instances are unit 
 * assembling the elements of a procurement process
 *
 * @author Pietro Terna 
 */
public class ProcurementAssembler extends SwarmObjectImpl implements 
    PTHistogramPlottable{


    /** the unit we are assembling for */
    Unit myUnit;
    /** the list of the order to be assembled with the procurements */
    public ListImpl waitingList;
    /** the list of end units */
    public ListImpl endUnitList;
    /** its iterator */
    public ListIndex endUnitListIndex;
    /** the vector of the quantities in the end units */
    public int [] quantitiesInEndUnits;
    /** the vector of the labels fo the end units */
    public int [] endUnitLabels;
    /** a procurement specification set */
    ProcurementSpecificationSet pendingProcurementSpecificationSet;

    /**
     * the constructor for ProcurementAssembler
     */
    public ProcurementAssembler (Zone aZone, ListImpl eul)
    {
	// Call the constructor for the parent class.
	super(aZone);
	int i;

	// the end unit list
	endUnitList = eul;
	// its iterator
	endUnitListIndex = endUnitList.listBegin(getZone());

	quantitiesInEndUnits = new int[endUnitList.getCount()];
	endUnitLabels = new int [endUnitList.getCount()];

	if(endUnitList.getCount()>0){
	    endUnitListIndex.setOffset(0);
	    for (i=0; i<endUnitList.getCount(); i++){
		endUnitLabels[i]= ((EndUnit) endUnitListIndex.get()).
		    getUnitNumber();
		endUnitListIndex.next();
	    }
	}

	// the list procurement processes to be assembled
	waitingList = new ListImpl (getZone());

    }

    /** setting the unit we are assembling for */
    public void setUnit(Unit u){
	myUnit = u;
    }

    /** adding an order to the waitingList */
    public boolean setProcurementWaitingList(Order anOrder){
	waitingList.addLast(anOrder);
	return true;
    }

    /** verifying procurements and eliminating the orders from our
     *  waitingList */
    public void checkingProcurementsAndFreeingOrders(){
	int n, i, ii, nmax;
	boolean completed;
	Order anOrder;

	nmax = waitingList.getCount();
	if(nmax>0)
	    for (n=0;n<nmax;n++)
		{

		    anOrder = (Order) waitingList.removeFirst();

	if(StartVEFrame.verbose)
		    System.out.println(" ");
		    // create the table of the quantities into the end units
		    for (i=0; i<endUnitList.getCount();i++){
			quantitiesInEndUnits[i]=
			    ((EndUnit) endUnitList.atOffset(i)).
			    getTemporaryListCount(anOrder.getOrderLayer());
	if(StartVEFrame.verbose)
			System.out.print("eu " + endUnitLabels[i] + " " +
					 quantitiesInEndUnits[i] + "   ");
		    }
	if(StartVEFrame.verbose)
		    System.out.println(
				   " as seen in procurementAssembler of unit "
				   + myUnit.getUnitNumber());

		    pendingProcurementSpecificationSet = 
			(ProcurementSpecificationSet)
			 anOrder.getPendingProcurementSpecificationSet();

		    if(pendingProcurementSpecificationSet == null){
			System.out.println("Internal error in" +
				       " procurementAssembler of unit " +
				       myUnit.getUnitNumber() +
				       " - order # " + 
				       anOrder.getOrderNumber() +  
				       "waiting for procurement "+
				       "has no procurement specifications.");
			MyExit.exit(1);
		    }

	if(StartVEFrame.verbose)
		    {
		    System.out.print("proc. spec. of order " +
				       pendingProcurementSpecificationSet.
				       getNumber() + " at pos. " +
				       pendingProcurementSpecificationSet.
				       getPositionInRecipe() + " items ");
		    for (i=0;i<pendingProcurementSpecificationSet.
			     getNumberOfItemsToBeProcured();i++)
			System.out.print(pendingProcurementSpecificationSet.
					 getItemToBeProcured(i) + " ");
		    System.out.println(" ");
			}

		    // checking endUnits for the required procurements
		    for (i=0;i<pendingProcurementSpecificationSet.
			     getNumberOfItemsToBeProcured();i++)
			for (ii=0; ii<endUnitList.getCount(); ii++)
			    if(endUnitLabels[ii]==
			       pendingProcurementSpecificationSet.
			       getItemToBeProcured(i))
				quantitiesInEndUnits[ii]--;
	if(StartVEFrame.verbose)
		    for(ii=0; ii<endUnitList.getCount(); ii++)
			System.out.print("eu " + endUnitLabels[ii] + " " +
					 quantitiesInEndUnits[ii] + "   ");
	if(StartVEFrame.verbose)
		    System.out.println(
				" as residual in procurementAssembler of unit "
				+ myUnit.getUnitNumber());

		    completed=true;
		    for (ii=0; ii<endUnitList.getCount(); ii++)
			if(quantitiesInEndUnits[ii]<0)completed=false;

		    if(completed)
			{
	if(StartVEFrame.verbose)
			    System.out.println("completed");
			    for (i=0;i<pendingProcurementSpecificationSet.
				     getNumberOfItemsToBeProcured();i++)
				for (ii=0; ii<endUnitList.getCount(); ii++)
				    if(endUnitLabels[ii]==
				       pendingProcurementSpecificationSet.
				       getItemToBeProcured(i))
					// set the procured items into
					// anOrder
					anOrder.setProcuredItemList((Order)
					((EndUnit) endUnitList.atOffset(ii)).
					    removeFromTemporaryList(
						   anOrder.getOrderLayer()) );
			}

		    else waitingList.addLast(anOrder);

		}
    }

    /**
     * return the waiting list length
     */
    public int getWaitingListLength()
    {
	return waitingList.getCount();
    }

    /**
     * checking if an order is in the waitingList
     */
    public boolean thisOrderIsInTheWaitingList(Order o){

	// the result is true if o is a member of waitingList; false in
	// the opposite case
	return waitingList.contains(o);
    }


    /**
     * PTHistogramPlottable interface method: getLabel() 
     */
    public String getLabel(){
	return "";
    }

    /**
     * PTHistogramPlottable interface method: getValueToPlot() 
     */
    public double getValueToPlot(){
	return (double) getWaitingListLength();
    }

}



